/*
 * SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#pragma once

#include <QObject>

class TextItemTest : public QObject
{
    Q_OBJECT
public:
    explicit TextItemTest(QObject *parent = nullptr);
    ~TextItemTest() override = default;

private Q_SLOTS:
    void shouldHaveDefaultValues();
    void shouldSerializeDefaultValue();
    void shouldTestTransformations();
    void shouldTestIntersects();
    void shouldRoundTrip();
    void shouldTestMode();
    void shouldTestCaretAndSelection();
    void shouldTestTextManipulation();
    void shouldTestLineRange();
    void shouldTestBreaks();
    void shouldTestNeedsCaching();
    void shouldTestLockAspectRatioWhenResizing();
};
