// SPDX-FileCopyrightText: 2025 Prayag Jain <prayagjain2@gmail.com>
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "linetool.hpp"

#include "item/factory/linefactory.hpp"
#include "linedrawingtool.hpp"
#include <KLocalizedString>
using namespace Qt::Literals::StringLiterals;

LineTool::LineTool(ApplicationContext *context)
    : LineDrawingTool(context)
{
    m_itemFactory = std::make_unique<LineFactory>();
}

QString LineTool::icon() const
{
    return u"tool_line"_s;
}

Tool::Type LineTool::type() const
{
    return Tool::Type::Line;
}
