// SPDX-FileCopyrightText: 2025 Prayag Jain <prayagjain2@gmail.com>
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "drawingtool.hpp"

DrawingTool::DrawingTool(ApplicationContext *context)
    : Tool(context)
{
}
