/*
 * SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */
#include "rectangleserializer.hpp"
#include "item/rectangle.hpp"

using namespace Qt::Literals::StringLiterals;
RectangleSerializer::RectangleSerializer(const RectangleItem *item)
    : PolygonSerializer(item)
{
}

RectangleSerializer::~RectangleSerializer() = default;
