/*
 * SPDX-FileCopyrightText: 2026 Laurent Montel <montel@kde.org>
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include "libdrawywidgets_export.h"
#include <KAboutData>
class LIBDRAWYWIDGETS_EXPORT AboutData : public KAboutData
{
public:
    AboutData();
};
