// SPDX-FileCopyrightText: 2025 Prayag Jain <prayagjain2@gmail.com>
//
// SPDX-License-Identifier: GPL-3.0-or-later

#pragma once

class ApplicationContext;

namespace Common
{
void renderCanvas(ApplicationContext *context);
};
